/// Random keygen  form [rusty-keys](https://github.com/rideron89/rusty-keys)
///```rust
/// let random = RandomKey::default();
/// let tkn0 = random.from_template("uuid",1)[0];
/// let tkn1 = random.get_key();
/// let tkn = format!("{}\n{}\n",tkn0,&tkn1);
///```  
/// `RandomKey` can be configured from Config (config.toml) with values for
/// -  length: `config.auth.keys_length`,
/// -  `alfanumeric_only`: `config.auth.keys_alfanumeric`,
///
#[derive(Debug, Clone)]
pub struct RandomKey {
  count: u32,
  length: u32,
  alfanumeric_only: bool,
}
impl Default for RandomKey {
  fn default() -> Self {
    Self {
      count: 1,
      length: 16,
      alfanumeric_only: false,
    }
  }
}
impl RandomKey {
  #[allow(clippy::missing_docs_in_private_items)]
  /// Configure `RandomKey`
  #[must_use]
  pub fn config(length: u32, alfanumeric_only: bool) -> Self {
    Self {
      count: 1,
      length,
      alfanumeric_only,
    }
  }
  #[allow(clippy::missing_docs_in_private_items)]
  /// Get a key
  #[must_use]
  pub fn get_key(&self) -> String {
    match self
      .get_list(self.length, self.count, self.alfanumeric_only)
      .get(0)
    {
      Some(key) => key.to_owned(),
      None => String::from(""),
    }
  }
  /// Get a list of keys
  #[must_use]
  #[allow(
    clippy::missing_docs_in_private_items,
    clippy::cast_precision_loss,
    clippy::cast_sign_loss,
    clippy::cast_possible_truncation
  )]
  pub fn get_list(&self, length: u32, count: u32, allow_symbols: bool) -> Vec<String> {
    let mut alphabet =
      String::from("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNPQRSTUVWXYZ0123456789");

    if allow_symbols {
      alphabet.push_str("/.;!@$");
    }
    let mut codes: Vec<String> = Vec::new();
    for _ in 0..count {
      let mut code = String::new();

      for _ in 0..length {
        let number = rand::random::<f32>() * (alphabet.len() as f32);
        let number = number.round() as usize;
        if let Some(character) = alphabet.chars().nth(number) {
          code.push(character)
        }
      }
      codes.push(code);
    }
    codes
  }
  /// Print a list of passwords for a given template.
  #[must_use]
  pub fn from_template(&self, template: &str, count: u32) -> Vec<String> {
    let mut codes: Vec<String> = Vec::new();
    for _ in 0..count {
      match template {
        "uuid" => codes.push(self.generate_uuid()),
        &_ => {
          // the process should never get to this point, because we are checking
          // against valid templates in a previous step
          eprintln!("'{}' is an invalid template", template);
        }
      }
    }
    codes
  }
  /// Generate a UUID key.
  /// Format: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
  /// Example: c17247f6-55b6-9226-28fc-952b7becc6e9
  #[must_use]
  #[allow(
    clippy::unnecessary_cast,
    clippy::cast_sign_loss,
    clippy::cast_possible_truncation
  )]
  pub fn generate_uuid(&self) -> String {
    let alphabet = String::from("abcdef0123456789");

    let mut code = String::new();

    for _ in 0..32 {
      // 16 is the length of the above Hex alphabet
      let number = rand::random::<f32>() * (16 as f32);
      let number = number.round() as usize;

      if let Some(character) = alphabet.chars().nth(number) {
        code.push(character)
      }
    }

    code.insert(20, '-');
    code.insert(16, '-');
    code.insert(12, '-');
    code.insert(8, '-');

    code
  }
}