/*! Utility functions for variety of tasks */
//
// Copyright 2020, Jesús Pérez Lorenzo

use anyhow::anyhow;
use regex::Regex;
use std::fs;
use std::process::Command;

#[derive(PartialEq, Default, Clone, Debug)]
struct Commit {
  hash: String,
  message: String,
}
/// A very simple Operating System Command runner capturing output
/// Allows two arguments, they can be empty
///```rust
/// let res =
/// match run_command("pwd", "", "") {
///   Ok(_) => format!("{} {}", "pwd","Done"),
///   Err(e) => format!("{}",e),
/// };
/// println!("Result: {}",res);
/// ```
#[allow(dead_code)]
#[allow(clippy::missing_errors_doc, clippy::filter_map)]
pub fn run_command(cmd: &str, arg1: &str, arg2: &str) -> anyhow::Result<()> {
  let output = match arg2 {
    "" => match arg1 {
      "" => Command::new(cmd).output()?,
      _ => Command::new(cmd).arg(arg1).output()?,
    },
    _ => Command::new(cmd).arg(arg1).arg(arg2).output()?,
  };

  if !output.status.success() {
    //dbg!(&output);
    let code = match output.status.code() {
      Some(val) => val,
      None => -1,
    };
    let stderr = String::from_utf8(output.stderr)?;
    let msg = format!(
      "Command {} executed with failing errors({}): {}",
      &cmd, &code, &stderr
    );
    println!("{}", &msg);
    return Err(anyhow!(msg));
    // error_chain::bail!("Command {} executed with failing error code", &cmd);
  }

  let pattern = Regex::new(
    r"(?x)
                             ([0-9a-fA-F]+) # commit hash
                             (.*)           # The commit message",
  )?;
  String::from_utf8(output.stdout)?
    .lines()
    .filter_map(|line| pattern.captures(line))
    .map(|cap| Commit {
      hash: cap[1].to_string(),
      message: cap[2].trim().to_string(),
    })
    .take(5)
    .for_each(|x| println!("{:?}", x));

  Ok(())
}
/// Trim newline
/// this function is critical for base64 content as usually adds a newline at the end, causing decode not work properly with code.
/// `trim_newline` should be called to clean content before to be decoded.
pub fn trim_newline(s: &mut String) {
  if s.ends_with('\n') {
    s.pop();
    if s.ends_with('\r') {
      s.pop();
    }
  }
}
/// Get last char from String
#[must_use]
pub fn get_last_char_num(text: &str) -> u32 {
  let v_last: Vec<char> = text.chars().rev().take(1).collect();
  let s_last: String = v_last.iter().collect();
  let mut num = 0;
  match s_last.parse::<u32>() {
    Ok(n) => num = n,
    Err(e) => println!("Error last_char_num: {}", e),
  }
  num
}
/// Check file permissions
/// Initial idea was capture file mode as 600 but permissions for unix in rust something else has to be done
/// ```rust
/// #[cfg(target_family = "unix")]
/// use std::os::unix::fs::PermissionsExt
/// ```
#[allow(clippy::missing_errors_doc)]
pub fn check_file_permissions(path: &str) -> anyhow::Result<bool> {
// let info = os_info::get();
//   // Print full information:
// println!("OS information: {}", info);

// // Print information separately:
// println!("Type: {}", info.os_type());
// println!("Version: {}", info.version());
// println!("Bitness: {}", info.bitness());
  // let os = os_type::current_platform();
  // let macos = os_type::OSType::OSX;
  // match os_type::current_platform().os_type {
  //   os_type::OSType::OSX => Ok(true),
  //   _ => {
  //     let perm = fs::File::open(path)?.metadata()?.permissions();
  //     Ok(perm.readonly())
  //   }
  // }
  let perm = fs::File::open(path)?.metadata()?.permissions();
  // if ! perm.readonly() {
  //    println!("Review permissions: {}", path);
  // }
  // Ok(true)
  Ok(perm.readonly())
}
